@@ -1,4 +1,7 @@
 import re
+import pypandoc
+import random
+from functools import partial
 import argparse
 import itertools
 import subprocess
@@ -43,6 +46,16 @@ def func_remove_milestone(options):
     version.remove_milestone(options.milestone)
     dump(options)
 
+ALL_EMOJIS=['sushi', 'rice', 'cookie', 'grapes', 'peach', 'pear', 'banana', 'cherries', 'watermelon', 'pizza', 'beer', 'cake', 'egg', 'green_apple', 'apple', 'icecream', 'meat_on_bone', 'hamburger', 'poultry_leg', 'dango', 'doughnut', 'ice_cream', 'birthday', 'candy', 'lollipop', 'lemon']
+def label_to_emoji(label, current_mapping={}, free_emojis=ALL_EMOJIS):
+    if label in current_mapping:
+        emoji = current_mapping[label]
+    else:
+        emoji = random.choice(free_emojis)
+        free_emojis.remove(emoji)
+        current_mapping[label] = emoji
+    return ':' + emoji + ':' + label
+
 def func_import_scope(options):
     jira_version = options.jira_version_template.format(version=options.version)
     logging.debug('jira_version = %s', jira_version)
@@ -56,7 +69,7 @@ def func_import_scope(options):
         sys.exit(EXIT_CODE_VERSION_NOT_FOUND)
     version = options.versions[options.version]
     if options.milestone not in version.get_milestones():
-        version.set_milestone(option.milestone, datetime.now().date())
+        version.set_milestone(options.milestone, datetime.now().date())
     scope_entries = [jira_issue_to_dict(x) for x in issues]
     version.get_milestones()[options.milestone].scope = scope_entries
     dump(options)
@@ -76,7 +89,7 @@ def func_sync_jira(options):
                 jira.create_version(project=options.jira_project, name=jira_version, description=version.description, releaseDate=version.release_date)
             continue
         jira_version_content = jira_versions[jira_version]
-        if jira_version_content.raw.get('description') != version.description:
+        if re.sub('^None$', '', jira_version_content.raw.get('description', '')) != version.description:
             logging.debug('updating description of version %s from "%s" to "%s"', version.tag, jira_version_content.raw.get('description'), version.description)
             if not options.dry_run:
                 jira_version_content.update(description=version.description)
@@ -102,6 +115,19 @@ def func_create_version(options):
     options.current_context['versions'].append(version)
     dump(options)
 
+def func_reorder_version(options):
+    try:
+        entry = next(i for i, x in enumerate(options.current_context['versions']) if x['tag'] == options.tag)
+    except StopIteration:
+        logging.error('no entry found with tag %s', options.tag)
+        sys.exit(EXIT_CODE_VERSION_NOT_FOUND)
+    versions = options.current_context['versions']
+    moving_version = versions[entry]
+    new_versions = versions[:entry] + versions[entry + 1:]
+    new_versions.insert(entry + int(options.places), moving_version)
+    options.current_context['versions'] = new_versions
+    dump(options)
+
 def func_remove_version(options):
     try:
         entry = next(x for x in options.current_context['versions'] if x['tag'] == options.tag)
@@ -186,6 +212,11 @@ def parse_arguments():
     parser_versions_set_description.add_argument('--description', required=True)
     parser_versions_set_description.set_defaults(func=func_set_description)
 
+    parser_versions_set_description = versions_subparser.add_parser('reorder', help='move vesion up or down')
+    parser_versions_set_description.add_argument('tag')
+    parser_versions_set_description.add_argument('places', help='how many places to move down (positive number) or up (negative number)')
+    parser_versions_set_description.set_defaults(func=func_reorder_version)
+
     report_parser = subparsers.add_parser('report')
     report_parser.add_argument('tag', nargs='?')
     report_parser.add_argument('--format', choices=['yaml', 'html', 'markdown'], default='html')
@@ -226,7 +257,7 @@ def get_version_status(options, tag, version):
             scope_status[content['ref']]['milestones'].append(ms['name'])
     status['scope_status'] = list(scope_status.values())
     for x in release_candidates_tags:
-        x['release_candidate_number'] = re.match('^.*-rc.([0-9]*)$', x['tag']).groups()[0]
+        x['release_candidate_number'] = int(re.match('^.*-rc.([0-9]*)$', x['tag']).groups()[0])
         x['status'] = 'rejected'
     if not release_candidates_tags:
         return status
@@ -246,7 +277,6 @@ def get_version_status(options, tag, version):
     return status
 
 
-HTML_TEMPLATE = Template(open(os.path.join(os.path.dirname(__file__), 'report.template.html')).read())
 MARKDOWN_TEMPLATE = Template(open(os.path.join(os.path.dirname(__file__), 'report.template.md')).read())
 
 def get_status(options):
@@ -255,14 +285,23 @@ def get_status(options):
     else:
         return [get_version_status(options, x.tag, x) for x in options.versions.values()]
 
+def content_link(ref, options):
+    if options.jira_baseurl:
+        return '[{ref}]({jira}/browse/{ref})'.format(jira=options.jira_baseurl.rstrip('/'), ref=ref)
+    else:
+        return ref
+
 def func_report(options):
     status = get_status(options)
     if options.format == 'yaml':
         yaml.dump(status, sys.stdout)
-    elif options.format == 'html':
-        sys.stdout.write(HTML_TEMPLATE.render(status=status))
-    elif options.format == 'markdown':
-        sys.stdout.write(MARKDOWN_TEMPLATE.render(status=status))
+    elif options.format in {'markdown', 'html'}:
+        markdown = MARKDOWN_TEMPLATE.render(status=status, options=options, content_link=content_link, partial=partial, map=map, label_to_emoji=label_to_emoji)
+        if options.format == 'markdown':
+            sys.stdout.write(markdown)
+        elif options.format == 'html':
+            html = pypandoc.convert_text(source=markdown, to='html', format='gfm', extra_args=('--include-in-header=' + os.path.realpath(os.path.join(os.path.dirname(__file__), 'github-pandoc.css')),))
+            sys.stdout.write(html)
 
 def main():
     options = parse_arguments()
@@ -301,7 +340,10 @@ class Version:
 
     @property
     def release_date(self):
-        return to_date(self.get_milestones().get('release'))
+        if 'release' not in self.get_milestones():
+            return None
+        else:
+            return self.get_milestones()['release'].date
 
     @release_date.setter
     def release_date(self, value):
